@@ -459,8 +459,8 @@ public class DeltaManager extends ClusterManagerBase {
         if (cluster.getMembers().length > 0) {
             SessionMessage msg = new SessionMessageImpl(getName(), SessionMessage.EVT_SESSION_CREATED, null, sessionId,
                     sessionId + "-" + System.currentTimeMillis());
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("deltaManager.sendMessage.newSession", name, sessionId));
+            if (log.isTraceEnabled()) {
+                log.trace(sm.getString("deltaManager.sendMessage.newSession", name, sessionId));
             }
             msg.setTimestamp(session.getCreationTime());
             counterSend_EVT_SESSION_CREATED++;
@@ -752,7 +752,7 @@ public class DeltaManager extends ClusterManagerBase {
      *                                   used
      */
     @Override
-    protected synchronized void startInternal() throws LifecycleException {
+    protected void startInternal() throws LifecycleException {
 
         super.startInternal();
 
@@ -863,8 +863,8 @@ public class DeltaManager extends ClusterManagerBase {
         if (mbr == null && log.isWarnEnabled()) {
             log.warn(sm.getString("deltaManager.noMasterMember", getName(), ""));
         }
-        if (mbr != null && log.isDebugEnabled()) {
-            log.debug(sm.getString("deltaManager.foundMasterMember", getName(), mbr));
+        if (mbr != null && log.isTraceEnabled()) {
+            log.trace(sm.getString("deltaManager.foundMasterMember", getName(), mbr));
         }
         return mbr;
     }
@@ -927,7 +927,7 @@ public class DeltaManager extends ClusterManagerBase {
      *                                   used
      */
     @Override
-    protected synchronized void stopInternal() throws LifecycleException {
+    protected void stopInternal() throws LifecycleException {
 
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("deltaManager.stopped", getName()));
@@ -1140,13 +1140,14 @@ public class DeltaManager extends ClusterManagerBase {
      * Expire all find sessions.
      */
     public void expireAllLocalSessions() {
-        long timeNow = System.currentTimeMillis();
         Session sessions[] = findSessions();
         int expireDirect = 0;
         int expireIndirect = 0;
 
-        if (log.isDebugEnabled()) {
-            log.debug("Start expire all sessions " + getName() + " at " + timeNow + " sessioncount " + sessions.length);
+        long timeNow = 0;
+        if (log.isTraceEnabled()) {
+            timeNow = System.currentTimeMillis();
+            log.trace("Start expire all sessions " + getName() + " at " + timeNow + " sessioncount " + sessions.length);
         }
         for (Session value : sessions) {
             if (value instanceof DeltaSession) {
@@ -1161,9 +1162,9 @@ public class DeltaManager extends ClusterManagerBase {
                 } // end if
             } // end if
         } // for
-        long timeEnd = System.currentTimeMillis();
-        if (log.isDebugEnabled()) {
-            log.debug("End expire sessions " + getName() + " expire processingTime " + (timeEnd - timeNow) +
+        if (log.isTraceEnabled()) {
+            long timeEnd = System.currentTimeMillis();
+            log.trace("End expire sessions " + getName() + " expire processingTime " + (timeEnd - timeNow) +
                     " expired direct sessions: " + expireDirect + " expired direct sessions: " + expireIndirect);
         }
     }
@@ -1190,8 +1191,8 @@ public class DeltaManager extends ClusterManagerBase {
 
             ClassLoader[] loaders = getClassLoaders();
             currentThread.setContextClassLoader(loaders[0]);
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("deltaManager.receiveMessage.eventType", getName(), msg.getEventTypeString(),
+            if (log.isTraceEnabled()) {
+                log.trace(sm.getString("deltaManager.receiveMessage.eventType", getName(), msg.getEventTypeString(),
                         sender));
             }
 
