@@ -43,7 +43,6 @@ import org.apache.catalina.ha.ClusterValve;
 import org.apache.catalina.ha.session.ClusterSessionListener;
 import org.apache.catalina.ha.session.DeltaManager;
 import org.apache.catalina.ha.session.JvmRouteBinderValve;
-import org.apache.catalina.ha.session.SessionMessage;
 import org.apache.catalina.tribes.Channel;
 import org.apache.catalina.tribes.ChannelListener;
 import org.apache.catalina.tribes.Member;
@@ -331,6 +330,14 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
         this.channelSendOptions = channelSendOptions;
     }
 
+    public void setChannelSendOptions(String channelSendOptions) {
+
+        int value = Channel.parseSendOptions(channelSendOptions);
+        if (value > 0) {
+            this.setChannelSendOptions(value);
+        }
+    }
+
     /**
      * has members
      */
@@ -385,6 +392,15 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
         return channelSendOptions;
     }
 
+    /**
+     * returns the SendOptions as a comma separated list of names for use by JMX
+     *
+     * @return a comma separated list of the option names
+     */
+    public String getChannelSendOptionsName() {
+        return Channel.getSendOptionsAsString(channelSendOptions);
+    }
+
     /**
      * Create new Manager without add to cluster (comes with start the manager)
      *
@@ -396,8 +412,7 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
     @Override
     public synchronized Manager createManager(String name) {
         if (log.isDebugEnabled()) {
-            log.debug("Creating ClusterManager for context " + name + " using class " +
-                    getManagerTemplate().getClass().getName());
+            log.debug(sm.getString("simpleTcpCluster.createManager", name, getManagerTemplate().getClass().getName()));
         }
         ClusterManager manager = null;
         try {
@@ -482,7 +497,6 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
      *
      * @see org.apache.catalina.ha.deploy.FarmWarDeployer#backgroundProcess()
      * @see org.apache.catalina.tribes.group.GroupChannel#heartbeat()
-     * @see org.apache.catalina.tribes.group.GroupChannel.HeartbeatThread#run()
      */
     @Override
     public void backgroundProcess() {
@@ -530,6 +544,8 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
             log.info(sm.getString("simpleTcpCluster.start"));
         }
 
+        channel.setUtilityExecutor(Container.getService(getContainer()).getServer().getUtilityExecutor());
+
         try {
             checkDefaults();
             registerClusterValve();
@@ -579,8 +595,8 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
         if (container != null) {
             for (Valve v : valves) {
                 ClusterValve valve = (ClusterValve) v;
-                if (log.isDebugEnabled()) {
-                    log.debug("Invoking addValve on " + getContainer() + " with class=" + valve.getClass().getName());
+                if (log.isTraceEnabled()) {
+                    log.trace("Invoking addValve on " + getContainer() + " with class=" + valve.getClass().getName());
                 }
                 if (valve != null) {
                     container.getPipeline().addValve(valve);
@@ -596,8 +612,8 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
     protected void unregisterClusterValve() {
         for (Valve v : valves) {
             ClusterValve valve = (ClusterValve) v;
-            if (log.isDebugEnabled()) {
-                log.debug("Invoking removeValve on " + getContainer() + " with class=" + valve.getClass().getName());
+            if (log.isTraceEnabled()) {
+                log.trace("Invoking removeValve on " + getContainer() + " with class=" + valve.getClass().getName());
             }
             if (valve != null) {
                 container.getPipeline().removeValve(valve);
@@ -634,6 +650,8 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
         } catch (Exception x) {
             log.error(sm.getString("simpleTcpCluster.stopUnable"), x);
         }
+
+        channel.setUtilityExecutor(null);
     }
 
 
@@ -679,13 +697,13 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
      */
     @Override
     public void send(ClusterMessage msg, Member dest) {
+        send(msg, dest, this.channelSendOptions);
+    }
+
+    @Override
+    public void send(ClusterMessage msg, Member dest, int sendOptions) {
         try {
             msg.setAddress(getLocalMember());
-            int sendOptions = channelSendOptions;
-            if (msg instanceof SessionMessage &&
-                    ((SessionMessage) msg).getEventType() == SessionMessage.EVT_ALL_SESSION_DATA) {
-                sendOptions = Channel.SEND_OPTIONS_SYNCHRONIZED_ACK | Channel.SEND_OPTIONS_USE_ACK;
-            }
             if (dest != null) {
                 if (!getLocalMember().equals(dest)) {
                     channel.send(new Member[] { dest }, msg, sendOptions);
@@ -697,7 +715,7 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
                 if (destmembers.length > 0) {
                     channel.send(destmembers, msg, sendOptions);
                 } else if (log.isDebugEnabled()) {
-                    log.debug("No members in cluster, ignoring message:" + msg);
+                    log.debug(sm.getString("simpleTcpCluster.noMembers", msg));
                 }
             }
         } catch (Exception x) {
@@ -777,8 +795,8 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
 
     public void messageReceived(ClusterMessage message) {
 
-        if (log.isDebugEnabled() && message != null) {
-            log.debug("Assuming clocks are synched: Replication for " + message.getUniqueId() + " took=" +
+        if (log.isTraceEnabled() && message != null) {
+            log.trace("Assuming clocks are synched: Replication for " + message.getUniqueId() + " took=" +
                     (System.currentTimeMillis() - (message).getTimestamp()) + " ms.");
         }
 
@@ -796,8 +814,7 @@ public class SimpleTcpCluster extends LifecycleMBeanBase
                 // Notify our interested LifecycleListeners
                 fireLifecycleEvent(RECEIVE_MESSAGE_FAILURE_EVENT, new SendMessageData(message, dest, null));
                 if (log.isDebugEnabled()) {
-                    log.debug("Message " + message.toString() + " from type " + message.getClass().getName() +
-                            " transferred but no listener registered");
+                    log.debug(sm.getString("simpleTcpCluster.noListener", message, message.getClass().getName()));
                 }
             }
         }
