@@ -329,9 +329,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * {@inheritDoc}
-     */
     @Override
     public void setId(String id, boolean notify) {
 
@@ -357,7 +354,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
     public void tellNew() {
 
         // Notify interested session event listeners
-        fireSessionEvent(Session.SESSION_CREATED_EVENT, null);
+        fireSessionEvent(SESSION_CREATED_EVENT, null);
 
         // Notify interested application event listeners
         Context context = manager.getContext();
@@ -795,7 +792,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
 
             // Notify interested session event listeners
             if (notify) {
-                fireSessionEvent(Session.SESSION_DESTROYED_EVENT, null);
+                fireSessionEvent(SESSION_DESTROYED_EVENT, null);
             }
 
             // Call the logout method
@@ -834,7 +831,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
     public void passivate() {
 
         // Notify interested session event listeners
-        fireSessionEvent(Session.SESSION_PASSIVATED_EVENT, null);
+        fireSessionEvent(SESSION_PASSIVATED_EVENT, null);
 
         // Notify ActivationListeners
         HttpSessionEvent event = null;
@@ -868,7 +865,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
         }
 
         // Notify interested session event listeners
-        fireSessionEvent(Session.SESSION_ACTIVATED_EVENT, null);
+        fireSessionEvent(SESSION_ACTIVATED_EVENT, null);
 
         // Notify ActivationListeners
         HttpSessionEvent event = null;
@@ -1344,8 +1341,9 @@ public class StandardSession implements HttpSession, Session, Serializable {
         // Call the valueBound() method if necessary
         if (notify && value instanceof HttpSessionBindingListener) {
             // Don't call any notification if replacing with the same value
+            // unless configured to do so
             Object oldValue = attributes.get(name);
-            if (value != oldValue) {
+            if (value != oldValue || manager.getNotifyBindingListenerOnUnchangedValue()) {
                 event = new HttpSessionBindingEvent(getSession(), name, value);
                 try {
                     ((HttpSessionBindingListener) value).valueBound(event);
@@ -1359,12 +1357,17 @@ public class StandardSession implements HttpSession, Session, Serializable {
         Object unbound = attributes.put(name, value);
 
         // Call the valueUnbound() method if necessary
-        if (notify && (unbound != null) && (unbound != value) && (unbound instanceof HttpSessionBindingListener)) {
-            try {
-                ((HttpSessionBindingListener) unbound).valueUnbound(new HttpSessionBindingEvent(getSession(), name));
-            } catch (Throwable t) {
-                ExceptionUtils.handleThrowable(t);
-                manager.getContext().getLogger().error(sm.getString("standardSession.bindingEvent"), t);
+        if (notify && unbound instanceof HttpSessionBindingListener) {
+            // Don't call any notification if replacing with the same value
+            // unless configured to do so
+            if (unbound != value || manager.getNotifyBindingListenerOnUnchangedValue()) {
+                try {
+                    ((HttpSessionBindingListener) unbound)
+                            .valueUnbound(new HttpSessionBindingEvent(getSession(), name));
+                } catch (Throwable t) {
+                    ExceptionUtils.handleThrowable(t);
+                    manager.getContext().getLogger().error(sm.getString("standardSession.bindingEvent"), t);
+                }
             }
         }
 
@@ -1384,12 +1387,14 @@ public class StandardSession implements HttpSession, Session, Serializable {
             HttpSessionAttributeListener listener = (HttpSessionAttributeListener) o;
             try {
                 if (unbound != null) {
-                    context.fireContainerEvent("beforeSessionAttributeReplaced", listener);
-                    if (event == null) {
-                        event = new HttpSessionBindingEvent(getSession(), name, unbound);
+                    if (unbound != value || manager.getNotifyAttributeListenerOnUnchangedValue()) {
+                        context.fireContainerEvent("beforeSessionAttributeReplaced", listener);
+                        if (event == null) {
+                            event = new HttpSessionBindingEvent(getSession(), name, unbound);
+                        }
+                        listener.attributeReplaced(event);
+                        context.fireContainerEvent("afterSessionAttributeReplaced", listener);
                     }
-                    listener.attributeReplaced(event);
-                    context.fireContainerEvent("afterSessionAttributeReplaced", listener);
                 } else {
                     context.fireContainerEvent("beforeSessionAttributeAdded", listener);
                     if (event == null) {
@@ -1402,7 +1407,9 @@ public class StandardSession implements HttpSession, Session, Serializable {
                 ExceptionUtils.handleThrowable(t);
                 try {
                     if (unbound != null) {
-                        context.fireContainerEvent("afterSessionAttributeReplaced", listener);
+                        if (unbound != value || manager.getNotifyAttributeListenerOnUnchangedValue()) {
+                            context.fireContainerEvent("afterSessionAttributeReplaced", listener);
+                        }
                     } else {
                         context.fireContainerEvent("afterSessionAttributeAdded", listener);
                     }
@@ -1460,8 +1467,8 @@ public class StandardSession implements HttpSession, Session, Serializable {
         principal = null; // Transient (may be set later)
         // setId((String) stream.readObject());
         id = (String) stream.readObject();
-        if (manager.getContext().getLogger().isDebugEnabled()) {
-            manager.getContext().getLogger().debug("readObject() loading session " + id);
+        if (manager.getContext().getLogger().isTraceEnabled()) {
+            manager.getContext().getLogger().trace("readObject() loading session " + id);
         }
 
         if (notes == null) {
@@ -1534,8 +1541,8 @@ public class StandardSession implements HttpSession, Session, Serializable {
                 }
                 throw wae;
             }
-            if (manager.getContext().getLogger().isDebugEnabled()) {
-                manager.getContext().getLogger().debug("  loading attribute '" + name + "' with value '" + value + "'");
+            if (manager.getContext().getLogger().isTraceEnabled()) {
+                manager.getContext().getLogger().trace("  loading attribute '" + name + "' with value '" + value + "'");
             }
             // Handle the case where the filter configuration was changed while
             // the web application was stopped.
@@ -1580,8 +1587,8 @@ public class StandardSession implements HttpSession, Session, Serializable {
         stream.writeObject(Boolean.valueOf(isValid));
         stream.writeObject(Long.valueOf(thisAccessedTime));
         stream.writeObject(id);
-        if (manager.getContext().getLogger().isDebugEnabled()) {
-            manager.getContext().getLogger().debug("writeObject() storing session " + id);
+        if (manager.getContext().getLogger().isTraceEnabled()) {
+            manager.getContext().getLogger().trace("writeObject() storing session " + id);
         }
 
         // Gather authentication information (if configured)
@@ -1638,8 +1645,8 @@ public class StandardSession implements HttpSession, Session, Serializable {
             stream.writeObject(saveNames.get(i));
             try {
                 stream.writeObject(saveValues.get(i));
-                if (manager.getContext().getLogger().isDebugEnabled()) {
-                    manager.getContext().getLogger().debug(
+                if (manager.getContext().getLogger().isTraceEnabled()) {
+                    manager.getContext().getLogger().trace(
                             "  storing attribute '" + saveNames.get(i) + "' with value '" + saveValues.get(i) + "'");
                 }
             } catch (NotSerializableException e) {
