@@ -17,8 +17,6 @@
 package org.apache.catalina.valves.rewrite;
 
 import java.io.BufferedReader;
-import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
@@ -37,12 +35,9 @@ import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
-import org.apache.catalina.Engine;
-import org.apache.catalina.Host;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.Pipeline;
-import org.apache.catalina.Valve;
 import org.apache.catalina.connector.Connector;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
@@ -53,6 +48,8 @@ import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.buf.UDecoder;
 import org.apache.tomcat.util.buf.UriUtil;
+import org.apache.tomcat.util.file.ConfigFileLoader;
+import org.apache.tomcat.util.file.ConfigurationSource;
 import org.apache.tomcat.util.http.RequestUtil;
 
 /**
@@ -155,29 +152,15 @@ public class RewriteValve extends ValveBase {
                     containerLog.debug(sm.getString("rewriteValve.readConfiguration", webInfResourcePath));
                 }
             }
-        } else if (getContainer() instanceof Host) {
-            String resourceName = getHostConfigPath(resourcePath);
-            File file = new File(getConfigBase(), resourceName);
+        } else {
+            String resourceName = Container.getConfigPath(getContainer(), resourcePath);
             try {
-                if (!file.exists()) {
-                    // Use getResource and getResourceAsStream
-                    is = getClass().getClassLoader()
-                        .getResourceAsStream(resourceName);
-                    if (is != null && containerLog.isDebugEnabled()) {
-                        containerLog.debug("Read configuration from CL at " + resourceName);
-                    }
-                } else {
-                    if (containerLog.isDebugEnabled()) {
-                        containerLog.debug("Read configuration from " + file.getAbsolutePath());
-                    }
-                    is = new FileInputStream(file);
+                ConfigurationSource.Resource resource = ConfigFileLoader.getSource().getResource(resourceName);
+                is = resource.getInputStream();
+            } catch (IOException e) {
+                if (containerLog.isDebugEnabled()) {
+                    containerLog.debug(sm.getString("rewriteValve.noConfiguration", resourceName), e);
                 }
-                if ((is == null) && (containerLog.isDebugEnabled())) {
-                    containerLog.debug("No configuration resource found: " + resourceName +
-                            " in " + getConfigBase() + " or in the classloader");
-                }
-            } catch (Exception e) {
-                containerLog.error("Error opening configuration", e);
             }
         }
 
@@ -336,12 +319,11 @@ public class RewriteValve extends ValveBase {
             boolean done = false;
             boolean qsa = false;
             boolean qsd = false;
-            boolean valveSkip = false;
             for (int i = 0; i < rules.length; i++) {
                 RewriteRule rule = rules[i];
                 CharSequence test = (rule.isHost()) ? host : urlDecoded;
                 CharSequence newtest = rule.evaluate(test, resolver);
-                if (newtest != null && !test.toString().equals(newtest.toString())) {
+                if (newtest != null && !test.equals(newtest.toString())) {
                     if (containerLog.isTraceEnabled()) {
                         containerLog.trace("Rewrote " + test + " as " + newtest
                                 + " with rule pattern " + rule.getPatternString());
@@ -363,10 +345,6 @@ public class RewriteValve extends ValveBase {
                     qsd = true;
                 }
 
-                if (!valveSkip && newtest != null && rule.isValveSkip()) {
-                    valveSkip = true;
-                }
-
                 // Final reply
 
                 // - forbidden
@@ -565,15 +543,7 @@ public class RewriteValve extends ValveBase {
                     pipeline.getFirst().invoke(request, response);
                 }
             } else {
-                Valve next = getNext();
-                if (valveSkip) {
-                    next = next.getNext();
-                    if (next == null) {
-                        // Ignore and invoke the next valve normally
-                        next = getNext();
-                    }
-                }
-                next.invoke(request, response);
+                getNext().invoke(request, response);
             }
 
         } finally {
@@ -583,51 +553,6 @@ public class RewriteValve extends ValveBase {
     }
 
 
-    /**
-     * @return config base.
-     */
-    protected File getConfigBase() {
-        File configBase =
-            new File(System.getProperty("catalina.base"), "conf");
-        if (!configBase.exists()) {
-            return null;
-        } else {
-            return configBase;
-        }
-    }
-
-
-    /**
-     * Find the configuration path where the rewrite configuration file
-     * will be stored.
-     * @param resourceName The rewrite configuration file name
-     * @return the full rewrite configuration path
-     */
-    protected String getHostConfigPath(String resourceName) {
-        StringBuffer result = new StringBuffer();
-        Container container = getContainer();
-        Container host = null;
-        Container engine = null;
-        while (container != null) {
-            if (container instanceof Host) {
-                host = container;
-            }
-            if (container instanceof Engine) {
-                engine = container;
-            }
-            container = container.getParent();
-        }
-        if (engine != null) {
-            result.append(engine.getName()).append('/');
-        }
-        if (host != null) {
-            result.append(host.getName()).append('/');
-        }
-        result.append(resourceName);
-        return result.toString();
-    }
-
-
     /**
      * This factory method will parse a line formed like:
      *
@@ -716,12 +641,17 @@ public class RewriteValve extends ValveBase {
                     }
                 }
                 if (tokenizer.hasMoreTokens()) {
-                    map.setParameters(tokenizer.nextToken());
+                    if (tokenizer.countTokens() == 1) {
+                        map.setParameters(tokenizer.nextToken());
+                    } else {
+                        List<String> params = new ArrayList<>();
+                        while (tokenizer.hasMoreTokens()) {
+                            params.add(tokenizer.nextToken());
+                        }
+                        map.setParameters(params.toArray(new String[0]));
+                    }
                 }
-                Object[] result = new Object[2];
-                result[0] = name;
-                result[1] = map;
-                return result;
+                return new Object[] { name, map };
             } else if (token.startsWith("#")) {
                 // it's a comment, ignore it
             } else {
@@ -865,8 +795,6 @@ public class RewriteValve extends ValveBase {
             }
             rule.setType(true);
             rule.setTypeValue(flag);
-        } else if (flag.startsWith("valveSkip") || flag.startsWith("VS")) {
-            rule.setValveSkip(true);
         } else {
             throw new IllegalArgumentException(sm.getString("rewriteValve.invalidFlags", line, flag));
         }
