@@ -16,11 +16,14 @@
  */
 package org.apache.coyote.http2;
 
+import java.io.File;
 import java.io.IOException;
 import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Set;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
 
 import javax.servlet.http.HttpServletResponse;
 
@@ -42,6 +45,7 @@ import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.parser.HttpParser;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 import org.apache.tomcat.util.net.DispatchType;
+import org.apache.tomcat.util.net.SendfileState;
 import org.apache.tomcat.util.net.SocketEvent;
 import org.apache.tomcat.util.net.SocketWrapperBase;
 import org.apache.tomcat.util.res.StringManager;
@@ -53,8 +57,11 @@ class StreamProcessor extends AbstractProcessor {
 
     private static final Set<String> H2_PSEUDO_HEADERS_REQUEST = new HashSet<>();
 
+    private final Lock processLock = new ReentrantLock();
     private final Http2UpgradeHandler handler;
     private final Stream stream;
+    private SendfileData sendfileData = null;
+    private SendfileState sendfileState = null;
 
     static {
         H2_PSEUDO_HEADERS_REQUEST.add(":method");
@@ -64,18 +71,18 @@ class StreamProcessor extends AbstractProcessor {
     }
 
     StreamProcessor(Http2UpgradeHandler handler, Stream stream, Adapter adapter, SocketWrapperBase<?> socketWrapper) {
-        super(socketWrapper.getEndpoint(), stream.getCoyoteRequest(), stream.getCoyoteResponse());
+        super(adapter, stream.getCoyoteRequest(), stream.getCoyoteResponse());
         this.handler = handler;
         this.stream = stream;
-        setAdapter(adapter);
         setSocketWrapper(socketWrapper);
     }
 
 
     final void process(SocketEvent event) {
         try {
-            // FIXME: the regular processor syncs on socketWrapper, but here this deadlocks
-            synchronized (this) {
+            // Note: The regular processor uses the socketWrapper lock, but using that here triggers a deadlock
+            processLock.lock();
+            try {
                 // HTTP/2 equivalent of AbstractConnectionHandler#process() without the
                 // socket <-> processor mapping
                 SocketState state = SocketState.CLOSED;
@@ -131,6 +138,8 @@ class StreamProcessor extends AbstractProcessor {
                         recycle();
                     }
                 }
+            } finally {
+                processLock.unlock();
             }
         } finally {
             handler.executeQueuedStream();
@@ -141,14 +150,35 @@ class StreamProcessor extends AbstractProcessor {
     @Override
     protected final void prepareResponse() throws IOException {
         response.setCommitted(true);
-        prepareHeaders(request, response, handler.getProtocol(), stream);
+        if (handler.hasAsyncIO() && handler.getProtocol().getUseSendfile()) {
+            prepareSendfile();
+        }
+        prepareHeaders(request, response, sendfileData == null, handler.getProtocol(), stream);
         stream.writeHeaders();
     }
 
 
+    private void prepareSendfile() {
+        String fileName =
+                (String) stream.getCoyoteRequest().getAttribute(org.apache.coyote.Constants.SENDFILE_FILENAME_ATTR);
+        if (fileName != null) {
+            sendfileData = new SendfileData();
+            sendfileData.path = new File(fileName).toPath();
+            sendfileData.pos = ((Long) stream.getCoyoteRequest()
+                    .getAttribute(org.apache.coyote.Constants.SENDFILE_FILE_START_ATTR)).longValue();
+            sendfileData.end =
+                    ((Long) stream.getCoyoteRequest().getAttribute(org.apache.coyote.Constants.SENDFILE_FILE_END_ATTR))
+                            .longValue();
+            sendfileData.left = sendfileData.end - sendfileData.pos;
+            sendfileData.stream = stream;
+        }
+    }
+
+
     // Static so it can be used by Stream to build the MimeHeaders required for
     // an ACK. For that use case coyoteRequest, protocol and stream will be null.
-    static void prepareHeaders(Request coyoteRequest, Response coyoteResponse, Http2Protocol protocol, Stream stream) {
+    static void prepareHeaders(Request coyoteRequest, Response coyoteResponse, boolean noSendfile,
+            Http2Protocol protocol, Stream stream) {
         MimeHeaders headers = coyoteResponse.getMimeHeaders();
         int statusCode = coyoteResponse.getStatus();
 
@@ -159,7 +189,7 @@ class StreamProcessor extends AbstractProcessor {
         // Compression can't be used with sendfile
         // Need to check for compression (and set headers appropriately) before
         // adding headers below
-        if (protocol != null && protocol.useCompression(coyoteRequest, coyoteResponse)) {
+        if (noSendfile && protocol != null && protocol.useCompression(coyoteRequest, coyoteResponse)) {
             // Enable compression. Headers will have been set. Need to configure
             // output filter at this point.
             stream.addOutputFilter(new GzipOutputFilter());
@@ -205,7 +235,10 @@ class StreamProcessor extends AbstractProcessor {
 
     @Override
     protected final void finishResponse() throws IOException {
-        stream.getOutputBuffer().end();
+        sendfileState = handler.processSendfile(sendfileData);
+        if (!(sendfileState == SendfileState.PENDING)) {
+            stream.getOutputBuffer().end();
+        }
     }
 
 
@@ -338,6 +371,18 @@ class StreamProcessor extends AbstractProcessor {
     }
 
 
+    @Override
+    protected boolean isTrailerFieldsReady() {
+        return stream.isTrailerFieldsReady();
+    }
+
+
+    @Override
+    protected boolean isTrailerFieldsSupported() {
+        return stream.isTrailerFieldsSupported();
+    }
+
+
     @Override
     protected Object getConnectionID() {
         return stream.getConnectionId();
@@ -365,7 +410,6 @@ class StreamProcessor extends AbstractProcessor {
         // Clear fields that can be cleared to aid GC and trigger NPEs if this
         // is reused
         setSocketWrapper(null);
-        setAdapter(null);
     }
 
 
@@ -399,7 +443,9 @@ class StreamProcessor extends AbstractProcessor {
             setErrorState(ErrorState.CLOSE_NOW, e);
         }
 
-        if (getErrorState().isError()) {
+        if (sendfileState == SendfileState.PENDING) {
+            return SocketState.SENDFILE;
+        } else if (getErrorState().isError()) {
             action(ActionCode.CLOSE, null);
             request.updateCounters();
             return SocketState.CLOSED;
@@ -466,17 +512,12 @@ class StreamProcessor extends AbstractProcessor {
         }
 
         // HTTP header names must be tokens.
+        // Stream#emitHeader() checks that all the pseudo headers appear first.
         MimeHeaders headers = request.getMimeHeaders();
-        boolean previousHeaderWasPseudoHeader = true;
         Enumeration<String> names = headers.names();
         while (names.hasMoreElements()) {
             String name = names.nextElement();
-            if (H2_PSEUDO_HEADERS_REQUEST.contains(name)) {
-                if (!previousHeaderWasPseudoHeader) {
-                    return false;
-                }
-            } else if (!HttpParser.isToken(name)) {
-                previousHeaderWasPseudoHeader = false;
+            if (!H2_PSEUDO_HEADERS_REQUEST.contains(name) && !HttpParser.isToken(name)) {
                 return false;
             }
         }
@@ -487,8 +528,8 @@ class StreamProcessor extends AbstractProcessor {
 
     @Override
     protected final boolean flushBufferedWrite() throws IOException {
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("streamProcessor.flushBufferedWrite.entry", stream.getConnectionId(),
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("streamProcessor.flushBufferedWrite.entry", stream.getConnectionId(),
                     stream.getIdAsString()));
         }
         if (stream.flush(false)) {
