@@ -71,11 +71,6 @@ public class TestRewriteValve extends TomcatBaseTest {
         doTestRewrite("RewriteRule ^(.*) $1", "/a/%255A", "/a/%255A");
     }
 
-    @Test
-    public void testNoopValveSkipRewrite() throws Exception {
-        doTestRewrite("RewriteRule ^(.*) $1 [VS]", "/a/%255A", "/a/%255A", null, null, true);
-    }
-
     @Test
     public void testPathRewrite() throws Exception {
         doTestRewrite("RewriteRule ^/b(.*) /a$1", "/b/%255A", "/a/%255A");
@@ -124,6 +119,12 @@ public class TestRewriteValve extends TomcatBaseTest {
                 "RewriteRule /b/.* /c/${mapa:${mapa:a}}", "/b/a.html", "/c/aaaa");
     }
 
+    @Test
+    public void testRewriteMap07() throws Exception {
+        doTestRewrite("RewriteMap mapa org.apache.catalina.valves.rewrite.TesterRewriteMapA foo bar\n" +
+                "RewriteRule /b/.* /c/${mapa:${mapa:a}}", "/b/a.html", "/c/aaaa");
+    }
+
     @Test
     public void testRewriteMap08() throws Exception {
         doTestRewrite("RewriteMap lc int:tolower\n" + "RewriteRule ^(.*) ${lc:$1}", "/C/AaA", "/c/aaa");
@@ -730,11 +731,6 @@ public class TestRewriteValve extends TomcatBaseTest {
 
     private void doTestRewrite(String config, String request, String expectedURI, String expectedQueryString,
             String expectedAttributeValue) throws Exception {
-        doTestRewrite(config, request, expectedURI, expectedQueryString, expectedAttributeValue, false);
-    }
-
-    private void doTestRewrite(String config, String request, String expectedURI, String expectedQueryString,
-            String expectedAttributeValue, boolean valveSkip) throws Exception {
 
         Tomcat tomcat = getTomcatInstance();
 
@@ -743,14 +739,6 @@ public class TestRewriteValve extends TomcatBaseTest {
 
         RewriteValve rewriteValve = new RewriteValve();
         ctx.getPipeline().addValve(rewriteValve);
-        if (valveSkip) {
-            ctx.getPipeline().addValve(new ValveBase() {
-                @Override
-                public void invoke(Request request, Response response) throws IOException, ServletException {
-                    throw new IllegalStateException();
-                }
-            });
-        }
 
         rewriteValve.setConfiguration(config);
 
